(* ::Package:: *)

(*:Package Version: 2.10 *)

(*:Mathematica Version: 8.01 *)

(*:Version History: 1.01 (1999.07.30 - 2006.04.08)
                    1.02 (2006.04.09 - 2007.08.14)
                    1.03 (2007.08.15 - 2007.08.18)
                    1.04 (2007.08.19 - 2012.02.18)
                    2.01 (2012.02.19 - 2016.04.09)
                    2.02 (2016.04.10 - 2017.08.24)CoulombWave
                    2.03 (2017.08.25 - 2017.08.28)GeneralSinIntegral
                    2.04 (2017.08.29 - 2017.10.09)+HyperErf.m
                    2.05 (2017.10.10 - 2018.06.01)GeneralErf, GeneralFresnel
                    2.06 (2018.06.02 - 2019.02.16)OwenT, MarcumQ
                    2.07 (2019.02.17 - 2021.01.29)Coulomb(eta=0)
                    2.08 (2021.01.30 - 2021.05.28)ParabolicCylinder, Hermite
                    2.09 (2021.05.29 - 2023.12.09)Laguerre, Hydrogen
                    2.10 (2023.12.10 - 20XX.XX.XX)Coulomb *)

(*:Copyright: Copyright 1999.07-2023.12, Souichiro Ikebe *)

(*:Author: Souichiro Ikebe *)

(*:Name: SpecialFunctions`KummerHypergeometric` *)

(*:Title: KummerHypergeometric, Whittaker, CoulombWave *)

(*:Keywords: WhittakerN, WhittakerMPrime, WhittakerWPrime, 
   HermiteH2, HermiteHPrime, HermiteH2Prime, 
   HermiteHNormalized, HermiteH2Normalized, 
   LaguerreL2, LaguerreLPrime, LaguerreL2Prime, 
   LaguerreLNormalized, LaguerreL2Normalized, 
   HydrogenElectronProbability, HydrogenElectronProbabilityR, 
   HydrogenElectronProbabilityCartesian, 
   ParabolicCylinderU, ParabolicCylinderV, ParabolicCylinderW, 
   ParabolicCylinderW1, ParabolicCylinderW2, 
   ParabolicCylinderWC, ParabolicCylinderWS, 
   ParabolicCylinderWEP, ParabolicCylinderWEM, 
   ParabolicCylinderUPrime, ParabolicCylinderVPrime, 
   ParabolicCylinderWPrime, ParabolicCylinderW1Prime, 
   ParabolicCylinderW2Prime, ParabolicCylinderWCPrime, 
   ParabolicCylinderWSPrime, ParabolicCylinderWEPPrime, 
   ParabolicCylinderWEMPrime, 
   Hypergeometric1G1, 
   CoulombWaveC, CoulombWaveSigma, CoulombWaveTheta, 
   CoulombWaveF, CoulombWaveFPrime, CoulombWaveG, 
   CoulombWaveGPrime, CoulombWaveH1, CoulombWaveH1Prime, 
   CoulombWaveH2, CoulombWaveH2Prime, 
   GeneralSinIntegralL, GeneralCosIntegralL, 
   GeneralSinIntegralS, GeneralCosIntegralS, 
   HyperErf, HyperErfi, HyperFresnelS, HyperFresnelC, 
   GeneralErf, GeneralErfi, GeneralFresnelS, GeneralFresnelC, 
   SinIntegralExp, CosIntegralExp, SinIntegralRecip, CosIntegralRecip, 
   GoodwinStatonG, GoodwinStatonGExp, FresnelF, FresnelG, FresnelPatternF, 
   ImprovedOwenT, ImprovedMarcumQ, 
   VoigtU, VoigtV, LineBroadeningH, HjertingH *)

(*:Summary:
    This package implements Whittaker's functions and 
    relation Kummer's confluent hypergeometric functions. *)

(**********************************************************************)



BeginPackage["SpecialFunctions`KummerHypergeometric`"]



WhittakerN::usage =
"WhittakerN[k,m,z] gives the Whittaker's functions
 of second kind N k,m(z)."

WhittakerMPrime::usage =
"WhittakerMPrime[k,m,z] gives the Whittaker's
 functions derivative of z."

WhittakerWPrime::usage =
"WhittakerWPrime[k,m,z] gives the Whittaker's
 functions derivative of z."

HermiteH2::usage =
"HermiteH2[v,z] gives the Hermite's functions
 of second kind."

HermiteHPrime::usage =
"HermiteHPrime[v,z] gives the Hermite's functions
 of first kind derivative of z."

HermiteH2Prime::usage =
"HermiteH2Prime[v,z] gives the Hermite's functions
 of second kind derivative of z."

HermiteHNormalized::usage =
"HermiteHNormalized[v,z] gives the normalized
 Hermite's functions."

HermiteH2Normalized::usage =
"HermiteH2Normalized[v,z] gives the normalized
 Hermite's functions of second kind."

LaguerreL2::usage =
"LaguerreL2[v,a,z] gives the Laguerre's functions
 of second kind."

LaguerreLPrime::usage =
"LaguerreLPrime[v,a,z] gives the Laguerre's functions
 of first kind derivative of z."

LaguerreL2Prime::usage =
"LaguerreL2Prime[v,a,z] gives the Laguerre's functions
 of second kind derivative of z."

LaguerreLNormalized::usage =
"LaguerreLNormalized[v,a,z] gives the normalized
 Laguerre's functions."

LaguerreL2Normalized::usage =
"LaguerreL2Normalized[v,a,z] gives the normalized
 Laguerre's functions of second kind."

HydrogenElectronProbability::usage =
"HydrogenElectronProbability[{n,l,m},r,th,ph,{z,a}] gives
 solusions of the Schroedinger equation of hydrogen-like atom in
 quantum mechanics : psi[{n,l,m},r,th,ph,{z,a}].
 Where, n: principal quantum number, l: azimuthal quantum number,
 m: magnetic quantum number, z: atomic number, a: Bohr radius,
 with n=1,2,3,...; l=0,1,2,...,n-1; m=-l,...,0,...,l; z=1,2,3,... .
 In particular, psi[{n,l,m},r,th,ph]=psi[{n,l,m},r,th,ph,{1,1}] or
 psi[{n,l,m},r,th,ph,{z}]=psi[{n,l,m},r,th,ph,{z,1}]."

HydrogenElectronProbabilityR::usage =
"HydrogenElectronProbabilityR[{n,l},r,{z,a}] gives radial
 solusions of the Schroedinger equation of hydrogen-like atom in
 quantum mechanics : R[{n,l},r,{z,a}].
 Where, n: principal quantum number, l: azimuthal quantum number,
 z: atomic number, a: Bohr radius,
 with n=1,2,3,...; l=0,1,2,...,n-1; z=1,2,3,... .
 In particular, R[{n,l},r]=R[{n,l},r,{1,1}] or
 R[{n,l},r,{z}]=R[{n,l},r,{z,1}]."

HydrogenElectronProbabilityCartesian::usage =
"HydrogenElectronProbabilityCartesian[{n,l,m},x,y,z,{zN,a}] gives
 solusions of the Schroedinger equation of hydrogen-like atom in
 quantum mechanics : psic[{n,l,m},x,y,z,{zN,a}]= 
 psi[{n,l,m},Sqrt[x^2+y^2+z^2],ArcTan[Sqrt[x^2+y^2]/z],ArcTan[y/x],{zN,a}]."

ParabolicCylinderU::usage =
"ParabolicCylinderU[a,z] gives the parabolic
 cylinder functions of first kind."

ParabolicCylinderV::usage =
"ParabolicCylinderV[a,z] gives the parabolic
 cylinder functions of second kind."

ParabolicCylinderW::usage =
"ParabolicCylinderW[a,z] gives the parabolic
 cylinder W-functions (see, NIST 12.14)."

ParabolicCylinderW1::usage =
"ParabolicCylinderW1[a,z] gives the parabolic
 cylinder W-functions of first kind (see, NIST 12.14).
 W1(a,z)=W(a,z)/Sqrt(k), and k=Sqrt(1+Exp(2*Pi*a))-Exp(Pi*a)."

ParabolicCylinderW2::usage =
"ParabolicCylinderW2[a,z] gives the parabolic
 cylinder W-functions of second kind (see, NIST 12.14).
 W2(a,z)=Sqrt(k)*W(a,-z), and k=Sqrt(1+Exp(2*Pi*a))-Exp(Pi*a)."

ParabolicCylinderWC::usage =
"ParabolicCylinderWC[a,z] gives the parabolic
 cylinder W-functions of first (even) kind.
 Wc(a,z)=Sqrt(k)*(W(a,-z)+W(a,z)), 
 and k=Sqrt(1+Exp(2*Pi*a))-Exp(Pi*a)."

ParabolicCylinderWS::usage =
"ParabolicCylinderWS[a,z] gives the parabolic
 cylinder W-functions of second (odd) kind.
 Ws(a,z)=Sqrt(k)*(W(a,-z)-W(a,z)), 
 and k=Sqrt(1+Exp(2*Pi*a))-Exp(Pi*a)."

ParabolicCylinderWEP::usage =
"ParabolicCylinderWEP[a,z] gives the parabolic
 cylinder W-functions of third (exp(iz) type) kind.
 Wep(a,z)=Wc(a,z)+i*Ws(a,z)."

ParabolicCylinderWEM::usage =
"ParabolicCylinderWEM[a,z] gives the parabolic
 cylinder W-functions of fourth (exp(-iz) type) kind.
 Wem(a,z)=Wc(a,z)-i*Ws(a,z)."

ParabolicCylinderUPrime::usage =
"ParabolicCylinderUPrime[a,z] gives the parabolic
 cylinder functions of first kind derivative of z."

ParabolicCylinderVPrime::usage =
"ParabolicCylinderVPrime[a,z] gives the parabolic
 cylinder functions of second kind derivative of z."

ParabolicCylinderWPrime::usage =
"ParabolicCylinderWPrime[a,z] gives the parabolic
 cylinder W-functions derivative of z."

ParabolicCylinderW1Prime::usage =
"ParabolicCylinderW1Prime[a,z] gives the parabolic
 cylinder W-functions of first kind derivative of z."

ParabolicCylinderW2Prime::usage =
"ParabolicCylinderW2Prime[a,z] gives the parabolic
 cylinder W-functions of second kind derivative of z."

ParabolicCylinderWCPrime::usage =
"ParabolicCylinderWCPrime[a,z] gives the parabolic
 cylinder W-functions of first (even) kind derivative of z."

ParabolicCylinderWSPrime::usage =
"ParabolicCylinderWSPrime[a,z] gives the parabolic
 cylinder W-functions of second (odd) kind derivative of z."

ParabolicCylinderWEPPrime::usage =
"ParabolicCylinderWEPPrime[a,z] gives the parabolic cylinder 
 W-functions of third (exp(iz) type) kind derivative of z."

ParabolicCylinderWEMPrime::usage =
"ParabolicCylinderWEMPrime[a,z] gives the parabolic cylinder 
 W-functions of fourth (exp(-iz) type) kind derivative of z."

Hypergeometric1G1::usage =
"Hypergeometric1G1[a,c,z] gives the Kummer's
 confluent hypergeometric functions of second kind."

CoulombWaveC::usage =
"CoulombWaveC[l,eta] gives coefficient C(l,eta) of the
 Coulomb wave function."

CoulombWaveSigma::usage =
"CoulombWaveSigma[l,eta] gives coefficient sigma(l,eta) of the
 Coulomb wave function."

CoulombWaveTheta::usage =
"CoulombWaveTheta[l,eta,rho] gives the Coulomb phase functions
 theta(l,eta,rho)."

CoulombWaveF::usage =
"CoulombWaveF[l,eta,rho] gives the Coulomb wave functions
 of first kind F(l,eta,rho)."

CoulombWaveFPrime::usage =
"CoulombWaveFPrime[l,eta,rho] gives the Coulomb wave
 functions of first kind derivative of rho."

CoulombWaveG::usage =
"CoulombWaveG[l,eta,rho] gives the Coulomb wave functions
 of second kind G(l,eta,rho)."

CoulombWaveGPrime::usage =
"CoulombWaveGPrime[l,eta,rho] gives the Coulomb wave
 functions of second kind derivative of rho."

CoulombWaveH1::usage =
"CoulombWaveH1[l,eta,rho] gives the Hankel-Coulomb wave functions
 of first kind H+(l,eta,rho)."

CoulombWaveH1Prime::usage =
"CoulombWaveH1Prime[l,eta,rho] gives the Hankel-Coulomb wave
 functions of first kind derivative of rho."

CoulombWaveH2::usage =
"CoulombWaveH2[l,eta,rho] gives the Hankel-Coulomb wave functions
 of second kind H-(l,eta,rho)."

CoulombWaveH2Prime::usage =
"CoulombWaveH2Prime[l,eta,rho] gives the Hankel-Coulomb wave
 functions of second kind derivative of rho."

GeneralSinIntegralL::usage =
"GeneralSinIntegralL[v,z] gives the Generalized sine integral
 functions : Si(v,z)=Integrate[t^(v-1)*Sin[t],{t,0,z}]."

GeneralCosIntegralL::usage =
"GeneralCosIntegralL[v,z] gives the Generalized cosine integral
 functions : Ci(v,z)=Integrate[t^(v-1)*Cos[t],{t,0,z}]."

GeneralSinIntegralS::usage =
"GeneralSinIntegralS[v,z] gives the Generalized sine integral
 functions : si(v,z)=Integrate[t^(v-1)*Sin[t],{t,z,Infinity}]."

GeneralCosIntegralS::usage =
"GeneralCosIntegralS[v,z] gives the Generalized cosine integral
 functions : ci(v,z)=Integrate[t^(v-1)*Cos[t],{t,z,Infinity}]."

HyperErf::usage =
"HyperErf[v,z] gives the hyper error function 
 Erf(v,z)=Integrate[Exp[-t^v],{t,0,z}]/Gamma[1+1/v]."

HyperErfi::usage =
"HyperErfi[v,z] gives the imaginary hyper error function
 Erfi(v,z)=Integrate[Exp[t^v],{t,0,z}]/Gamma[1+1/v]."

GeneralErf::usage =
"GeneralErf[v,m,z] gives the Generalized error function
 Erf(v,m,z)=Integrate[t^m*Exp[-t^v],{t,0,z}]."

GeneralErfi::usage =
"GeneralErfi[v,m,z] gives the imaginary Generalized error function
 Erfi(v,m,z)=Integrate[t^m*Exp[t^v],{t,0,z}]."

HyperFresnelS::usage =
"HyperFresnelS[v,z] gives the hyper Fresnel function
 FresnelS(v,z)=Integrate[Sin[Pi/2*t^v],{t,0,z}]."

HyperFresnelC::usage =
"HyperFresnelC[v,z] gives the hyper Fresnel function
 FresnelC(v,z)=Integrate[Cos[Pi/2*t^v],{t,0,z}]."

GeneralFresnelS::usage =
"GeneralFresnelS[v,m,z] gives the Generalized Fresnel function
 FresnelS(v,m,z)=Integrate[t^m*Sin[t^v],{t,0,z}]."

GeneralFresnelC::usage =
"GeneralFresnelC[v,m,z] gives the Generalized Fresnel function
 FresnelC(v,m,z)=Integrate[t^m*Cos[t^v],{t,0,z}]."

SinIntegralExp::usage =
"SinIntegralExp[z] gives the sine integral function
 of argument Exp[z].
 SinIntegralExp(z)=Integrate[Sin[Exp[t]],{t,z,Infinity}]."

CosIntegralExp::usage =
"CosIntegralExp[z] gives the cosine integral function
 of argument Exp[z].
 CosIntegralExp(z)=Integrate[Cos[Exp[t]],{t,z,Infinity}]."

SinIntegralRecip::usage =
"SinIntegralRecip[z] gives the sine integral function
 of argument 1/z.
 SinIntegralRecip(z)=Integrate[Sin[1/t],{t,0,z}]."

CosIntegralRecip::usage =
"CosIntegralRecip[z] gives the cosine integral function
 of argument 1/z.
 CosIntegralRecip(z)=Integrate[Cos[1/t],{t,0,z}]."

GoodwinStatonG::usage =
"GoodwinStatonG[z] gives the Goodwin-Staton's integral function. 
 Integrate[Exp[-t^2]/(t+z),{t,0,Infinity}]."

GoodwinStatonGExp::usage =
"GoodwinStatonGExp[z] gives the Goodwin-Staton's integral function in 
 exponential function. 
 GoodwinStatonGExp[z]=GoodwinStatonG[Exp[z]]."

FresnelF::usage =
"FresnelF[z] gives the auxiliary Fresnel F-function."

FresnelG::usage =
"FresnelG[z] gives the auxiliary Fresnel G-function."

FresnelPatternF::usage =
"FresnelPatternF[z] gives the Fresnel interference pattern function."

ImprovedOwenT::usage =
"ImprovedOwenT[z,w] gives the value of the Owen's T-function.
 This function, improved the same function of Mathematica at large Abs(w)."

ImprovedMarcumQ::usage =
"ImprovedMarcumQ[m,z,w] gives the value of the Marcum Q-function.
 This function, improved the same function of Mathematica.
 (But, unsame w-branch cut of Mathematica.)"

VoigtU::usage =
"VoigtU[z,t] gives the Voigt U-function."

VoigtV::usage =
"VoigtV[z,t] gives the Voigt V-function."

LineBroadeningH::usage =
"LineBroadeningH[a,z] gives line broadening H-function.
 This function same the Voigt-plasma dispersion v(z,a)."

HjertingH::usage =
"HjertingH[n,z] gives the Hjerting function.
 This function related the Voigt-plasma dispersion v(z,a).
 (See, W.J.Thompson : Atlas for Computing Mathematical Functions, 
 John Wiley & Sons, Inc. 1997)"



Begin["SpecialFunctions`KummerHypergeometric`Private`"]



Attributes[WhittakerN]={Listable}

Attributes[WhittakerMPrime]={Listable}

Attributes[WhittakerWPrime]={Listable}

Attributes[HermiteH2]={Listable}

Attributes[HermiteHPrime]={Listable}

Attributes[HermiteH2Prime]={Listable}

Attributes[HermiteHNormalized]={Listable}

Attributes[HermiteH2Normalized]={Listable}

Attributes[LaguerreL2]={Listable}

Attributes[LaguerreLPrime]={Listable}

Attributes[LaguerreL2Prime]={Listable}

Attributes[LaguerreLNormalized]={Listable}

Attributes[LaguerreL2Normalized]={Listable}

(* Attributes[HydrogenElectronProbability]={Listable} *)

(* Attributes[HydrogenElectronProbabilityR]={Listable} *)

(* Attributes[HydrogenElectronProbabilityCartesian]={Listable} *)

Attributes[ParabolicCylinderU]={Listable}

Attributes[ParabolicCylinderV]={Listable}

Attributes[ParabolicCylinderW]={Listable}

Attributes[ParabolicCylinderW1]={Listable}

Attributes[ParabolicCylinderW2]={Listable}

Attributes[ParabolicCylinderWC]={Listable}

Attributes[ParabolicCylinderWS]={Listable}

Attributes[ParabolicCylinderWEP]={Listable}

Attributes[ParabolicCylinderWEM]={Listable}

Attributes[ParabolicCylinderUPrime]={Listable}

Attributes[ParabolicCylinderVPrime]={Listable}

Attributes[ParabolicCylinderWPrime]={Listable}

Attributes[ParabolicCylinderW1Prime]={Listable}

Attributes[ParabolicCylinderW2Prime]={Listable}

Attributes[ParabolicCylinderWCPrime]={Listable}

Attributes[ParabolicCylinderWSPrime]={Listable}

Attributes[ParabolicCylinderWEPPrime]={Listable}

Attributes[ParabolicCylinderWEMPrime]={Listable}

Attributes[Hypergeometric1G1]={Listable}

Attributes[CoulombWaveC]={Listable}

Attributes[CoulombWaveSigma]={Listable}

Attributes[CoulombWaveTheta]={Listable}

Attributes[CoulombWaveF]={Listable}

Attributes[CoulombWaveFPrime]={Listable}

Attributes[CoulombWaveG]={Listable}

Attributes[CoulombWaveGPrime]={Listable}

Attributes[CoulombWaveH1]={Listable}

Attributes[CoulombWaveH1Prime]={Listable}

Attributes[CoulombWaveH2]={Listable}

Attributes[CoulombWaveH2Prime]={Listable}

Attributes[GeneralSinIntegralL]={Listable}

Attributes[GeneralCosIntegralL]={Listable}

Attributes[GeneralSinIntegralS]={Listable}

Attributes[GeneralCosIntegralS]={Listable}

Attributes[HyperErf]={Listable}

Attributes[HyperErfi]={Listable}

Attributes[GeneralErf]={Listable}

Attributes[GeneralErfi]={Listable}

Attributes[HyperFresnelS]={Listable}

Attributes[HyperFresnelC]={Listable}

Attributes[GeneralFresnelS]={Listable}

Attributes[GeneralFresnelC]={Listable}

Attributes[SinIntegralExp]={Listable}

Attributes[CosIntegralExp]={Listable}

Attributes[SinIntegralRecip]={Listable}

Attributes[CosIntegralRecip]={Listable}

Attributes[GoodwinStatonG]={Listable}

Attributes[GoodwinStatonGExp]={Listable}

Attributes[FresnelF]={Listable}

Attributes[FresnelG]={Listable}

Attributes[FresnelPatternF]={Listable}

Attributes[ImprovedOwenT]={Listable}

Attributes[ImprovedMarcumQ]={Listable}

Attributes[VoigtU]={Listable}

Attributes[VoigtV]={Listable}

Attributes[LineBroadeningH]={Listable}

Attributes[HjertingH]={Listable}



(**********************************************************************)
(*                         WhittakerN[k,m,z]                          *)
(**********************************************************************)

WhittakerN[k_,m_,z_]:=
  Exp[-z/2] z^(1/2+m) Hypergeometric1G1[1/2+m-k,1+2 m,z];

(**********************************************************************)
(*                       WhittakerMPrime[k,m,z]                       *)
(**********************************************************************)

WhittakerMPrime[k_,m_,z_]:=
  ((z/2-k) WhittakerM[k,m,z]+(1/2+k+m) WhittakerM[k+1,m,z])/z;

(**********************************************************************)
(*                       WhittakerWPrime[k,m,z]                       *)
(**********************************************************************)

WhittakerWPrime[k_,m_,z_]:=
  ((z/2-k) WhittakerW[k,m,z]-WhittakerW[k+1,m,z])/z;



(**********************************************************************)
(*                           HermiteH2[v,z]                           *)
(**********************************************************************)

HermiteH2[v_,z_]:=
  Module[{vp,zp,prec=20},
    vp=SetPrecision[v,prec];
    zp=SetPrecision[z,prec];
    2^vp/Sqrt[Pi]*
      (2*Cos[Pi*vp/2]*Gamma[vp/2+1]*zp*Hypergeometric1F1[(1-vp)/2,3/2,zp^2]-
      Sin[Pi*vp/2]*Gamma[vp/2+1/2]*Hypergeometric1F1[-vp/2,1/2,zp^2])]/;
  (NumericQ[v] && NumericQ[z])

(**********************************************************************)
(*                         HermiteHPrime[v,z]                         *)
(**********************************************************************)

HermiteHPrime[v_,z_]:=
  2*v*HermiteH[v-1,z]/;
  (NumericQ[v] && NumericQ[z])

(**********************************************************************)
(*                        HermiteH2Prime[v,z]                         *)
(**********************************************************************)

HermiteH2Prime[v_,z_]:=
  2*v*HermiteH2[v-1,z]/;
  (NumericQ[v] && NumericQ[z])

(**********************************************************************)
(*                      HermiteHNormalized[v,z]                       *)
(**********************************************************************)

HermiteHNormalized[v_,z_]:=
  2^(-v/2)/Pi^(1/4)*Exp[-LogGamma[v+1]/2] Exp[-z^2/2] HermiteH[v,z]/;
  (NumericQ[v] && NumericQ[z])

(**********************************************************************)
(*                      HermiteH2Normalized[v,z]                      *)
(**********************************************************************)

HermiteH2Normalized[v_,z_]:=
  2^(-v/2)/Pi^(1/4)*Exp[-LogGamma[v+1]/2] Exp[-z^2/2] HermiteH2[v,z]/;
  (NumericQ[v] && NumericQ[z])



(**********************************************************************)
(*                         LaguerreL2[v,a,z]                          *)
(**********************************************************************)

(**********  Common code  **********)

chopIntQ[v_]:=
  (Chop[Im[v]]===0 && Chop[v-Floor[v+1/2]]===0);

vaRegQ1[v_,a_]:=
  (chopIntQ[v] && chopIntQ[a] && Re[v]<Re[-a] || 
   Not[chopIntQ[v] && chopIntQ[a]] && chopIntQ[v+a] && Re[v+a]<-0.1);

vaRegQ2[v_,a_]:=
  (chopIntQ[v] && chopIntQ[a] && Not[Re[v]<Re[-a]] && Re[v]>1 && Re[a]<0);

vaRegQ[v_,a_]:=
  (chopIntQ[v] && chopIntQ[a] && Not[vaRegQ1[v,a]] && Not[vaRegQ2[v,a]] || 
   Not[chopIntQ[v] && chopIntQ[a]] && Not[vaRegQ1[v,a]]);

(**********  LaguerreL2  **********)

LaguerreL2[v_,z_]:=LaguerreL2[v,0,z];

LaguerreL2[v_,a_,z_]:=
  ComplexInfinity/;(vaRegQ1[v,a])

LaguerreL2[v_,a_,z_]:=
  Module[{vp,ap,zp,prec=50,val},
    vp=SetPrecision[v,prec];
    ap=SetPrecision[a,prec];
    zp=SetPrecision[z,prec];
    val=(Cos[vp*Pi]*LaguerreL[vp,ap,zp]-
      HypergeometricU[-vp,ap+1,zp]/Gamma[vp+1])/Sin[vp*Pi];
    If[Abs[Im[v]]>0 || Abs[Im[a]]>0 || Abs[Im[z]]>0 || Re[z]<=0,
      val,Re[val]]]/;
  (vaRegQ[v,a] && Not[chopIntQ[v]] && 
   NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{ap,zp,prec=50,n=0,la0,la1,la},
    ap=SetPrecision[a,prec];
    zp=SetPrecision[z,prec];
    la0=LaguerreL2[0,ap,zp];
    la1=LaguerreL2[1,ap,zp];
    While[n>IntegerPart[Re[v]],
      la=((2 n+ap+1-zp)*la0-(n+1)*la1)/(n+ap);
      la1=la0;
      la0=la;
      n--];
    la]/;
  (vaRegQ[v,a] && chopIntQ[v] && Re[v]<-0.1 && 
   NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{ap,zp,prec=50,n=1,la0,la1,la},
    ap=SetPrecision[a,prec];
    zp=SetPrecision[z,prec];
    la0=LaguerreL2[0,ap,zp];
    la1=LaguerreL2[1,ap,zp];
    While[n<IntegerPart[Re[v]],
      la=((2 n+ap+1-zp)*la1-(n+ap)*la0)/(n+1);
      la0=la1;
      la1=la;
      n++];
    la]/;
  (vaRegQ[v,a] && chopIntQ[v] && Re[v]>1.1 && 
   NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{ap,zp,prec=50,val},
    ap=SetPrecision[a,prec];
    zp=SetPrecision[z,prec];
    val=-Cot[Pi*ap]-Gamma[ap+1]/Pi*(-zp)^ap*zp^(-ap)*Gamma[-ap,0,-zp];
    If[Abs[Im[v]]>0 || Abs[Im[a]]>0 || Abs[Im[z]]>0 || Re[z]<=0,
      val,Re[val]]]/;
  (vaRegQ[v,a] && chopIntQ[v] && Not[chopIntQ[a]] && -0.1<Re[v]<0.1 && 
   NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{ap,zp,prec=50,val},
    ap=SetPrecision[a,prec];
    zp=SetPrecision[z,prec];
    val=(zp-ap-1)*(Cot[Pi*ap]+Gamma[ap+1]/Pi*(-zp)^ap*zp^(-ap)*Gamma[-ap,0,-zp])-
      Gamma[ap+1]/Pi*zp^(-ap)*Exp[zp];
    If[Abs[Im[v]]>0 || Abs[Im[a]]>0 || Abs[Im[z]]>0 || Re[z]<=0,
      val,Re[val]]]/;
  (vaRegQ[v,a] && chopIntQ[v] && Not[chopIntQ[a]] && 
   0.9<Re[v]<1.1 && NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{vp,zp,prec=50,n=0,la0,la1,la},
    vp=SetPrecision[v,prec];
    zp=SetPrecision[z,prec];
    la0=LaguerreL2[vp,0,zp];
    la1=LaguerreL2[vp,1,zp];
    While[n>IntegerPart[Re[a]],
      la=((zp+n)*la0-zp*la1)/(vp+n);
      la1=la0;
      la0=la;
      n--];
    la]/;
  ((vaRegQ2[v,a] || vaRegQ[v,a] && chopIntQ[v] && chopIntQ[a] && 
   -0.1<Re[v]<1.1 && Re[a]<-0.1) && NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{vp,zp,prec=50,n=1,la0,la1,la},
    vp=SetPrecision[v,prec];
    zp=SetPrecision[z,prec];
    la0=LaguerreL2[vp,0,zp];
    la1=LaguerreL2[vp,1,zp];
    While[n<IntegerPart[Re[a]],
      la=((zp+n)*la1-(vp+n)*la0)/zp;
      la0=la1;
      la1=la;
      n++];
    la]/;
  (vaRegQ[v,a] && chopIntQ[v] && chopIntQ[a] && -0.1<Re[v]<1.1 && 
   Re[a]>1.1 && NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{zp},
    zp=SetPrecision[z,50];
    -ExpIntegralEi[zp]/Pi]/;
  (vaRegQ[v,a] && chopIntQ[v] && chopIntQ[a] && -0.1<Re[v]<0.1 && 
   -0.1<Re[a]<0.1 && NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{zp},
    zp=SetPrecision[z,50];
    ((zp-1)*ExpIntegralEi[zp]-Exp[zp])/Pi]/;
  (vaRegQ[v,a] && chopIntQ[v] && chopIntQ[a] && 0.9<Re[v]<1.1 && 
   -0.1<Re[a]<0.1 && NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{zp},
    zp=SetPrecision[z,50];
    (Exp[zp]/zp-ExpIntegralEi[zp])/Pi]/;
  (vaRegQ[v,a] && chopIntQ[v] && chopIntQ[a] && -0.1<Re[v]<0.1 && 
   0.9<Re[a]<1.1 && NumericQ[v] && NumericQ[a] && NumericQ[z])

LaguerreL2[v_,a_,z_]:=
  Module[{zp},
    zp=SetPrecision[z,50];
    ((zp-2)*ExpIntegralEi[zp]-(1-1/zp)*Exp[zp])/Pi]/;
  (vaRegQ[v,a] && chopIntQ[v] && chopIntQ[a] && 0.9<Re[v]<1.1 && 
   0.9<Re[a]<1.1 && NumericQ[v] && NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                       LaguerreLPrime[v,a,z]                        *)
(**********************************************************************)

LaguerreLPrime[v_,z_]:=LaguerreLPrime[v,0,z];

LaguerreLPrime[v_,a_,z_]:=
  -LaguerreL[v-1,a+1,z]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                       LaguerreL2Prime[v,a,z]                       *)
(**********************************************************************)

LaguerreL2Prime[v_,z_]:=LaguerreL2Prime[v,0,z];

LaguerreL2Prime[v_,a_,z_]:=
  -LaguerreL2[v-1,a+1,z]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                     LaguerreLNormalized[v,a,z]                     *)
(**********************************************************************)

LaguerreLNormalized[v_,z_]:=LaguerreLNormalized[v,0,z];

LaguerreLNormalized[v_,a_,z_]:=
  Exp[(LogGamma[v+1]-LogGamma[v+a+1])/2]*
  z^(a/2)*Exp[-z/2]*LaguerreL[v,a,z]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                    LaguerreL2Normalized[v,a,z]                     *)
(**********************************************************************)

LaguerreL2Normalized[v_,z_]:=LaguerreL2Normalized[v,0,z];

LaguerreL2Normalized[v_,a_,z_]:=
  Exp[(LogGamma[v+1]-LogGamma[v+a+1])/2]*
  z^(a/2)*Exp[-z/2]*LaguerreL2[v,a,z]/;
  (NumericQ[v] && NumericQ[a] && NumericQ[z])



(**********************************************************************)
(*         HydrogenElectronProbability[{n,l,m},r,th,ph,{z,a}]         *)
(**********************************************************************)

HydrogenElectronProbability[{n_,l_,m_},r_,th_,ph_]:=
  HydrogenElectronProbability[{n,l,m},r,th,ph,{1,1}];

HydrogenElectronProbability[{n_,l_,m_},r_,th_,ph_,{z_}]:=
  HydrogenElectronProbability[{n,l,m},r,th,ph,{z,1}];

HydrogenElectronProbability[{n_,l_,m_},r_,th_,ph_,{z_,a_}]:=
  z/a*Sqrt[2/(n^3*r)]*
    LaguerreLNormalized[n-l-1,2*l+1,2*z*r/(a*n)]*
    SphericalHarmonicY[l,m,th,ph]/;
  (NumericQ[r] && NumericQ[th] && NumericQ[ph] && NumericQ[a] && 
   IntegerQ[n] && IntegerQ[l] && IntegerQ[m] && IntegerQ[z] && 
   n>0 && 0<=l<n && Abs[m]<=l && z>0)

(**********************************************************************)
(*            HydrogenElectronProbabilityR[{n,l},r,{z,a}]             *)
(**********************************************************************)

HydrogenElectronProbabilityR[{n_,l_},r_]:=
  HydrogenElectronProbabilityR[{n,l},r,{1,1}];

HydrogenElectronProbabilityR[{n_,l_},r_,{z_}]:=
  HydrogenElectronProbabilityR[{n,l},r,{z,1}];

HydrogenElectronProbabilityR[{n_,l_},r_,{z_,a_}]:=
  z/a*Sqrt[2/(n^3*r)]*
    LaguerreLNormalized[n-l-1,2*l+1,2*z*r/(a*n)]/;
  (NumericQ[r] && NumericQ[a] && IntegerQ[n] && IntegerQ[l] && 
   IntegerQ[z] && n>0 && 0<=l<n && z>0)

(**********************************************************************)
(*     HydrogenElectronProbabilityCartesian[{n,l,m},x,y,z,{zN,a}]     *)
(**********************************************************************)

HydrogenElectronProbabilityCartesian[{n_,l_,m_},x_,y_,z_]:=
  HydrogenElectronProbabilityCartesian[{n,l,m},x,y,z,{1,1}];

HydrogenElectronProbabilityCartesian[{n_,l_,m_},x_,y_,z_,{zN_}]:=
  HydrogenElectronProbabilityCartesian[{n,l,m},x,y,z,{zN,1}];

HydrogenElectronProbabilityCartesian[{n_,l_,m_},x_,y_,z_,{zN_,a_}]:=
  HydrogenElectronProbability[{n,l,m},Sqrt[x^2+y^2+z^2],
    ArcTan[z,Sqrt[x^2+y^2]],ArcTan[x,y],{zN,a}];



(**********************************************************************)
(*                      ParabolicCylinderU[a,z]                       *)
(**********************************************************************)

ParabolicCylinderU[a_,z_]:=
  2^(a/2+1/4)*Exp[-z^2/4]*HermiteH[-a-1/2,z/Sqrt[2]]/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                      ParabolicCylinderV[a,z]                       *)
(**********************************************************************)

ParabolicCylinderV[a_,z_]:=
  Module[{val},
    val=I/Gamma[1/2-a]*ParabolicCylinderU[a,z]+
      Sqrt[2/Pi]*Exp[I*Pi*(a/2-1/4)]*ParabolicCylinderU[-a,-I*z];
    If[Abs[Im[a]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (Not[numIntegQ[a]] && NumericQ[a] && NumericQ[z])

ParabolicCylinderV[a_,z_]:=
  Module[{val},
    val=Gamma[a+1/2]/Pi*
      (Sin[Pi*a]*ParabolicCylinderU[a,z]+ParabolicCylinderU[a,-z]);
    If[Abs[Im[z]]>0,val,Re[val]]]/;
  (numIntegQ[a] && NumericQ[a] && NumericQ[z])

numIntegQ[a_]:=
  (Chop[Im[a]]===0 && Chop[a-Floor[a+1/2]]===0);

(**********************************************************************)
(*                      ParabolicCylinderW[a,z]                       *)
(**********************************************************************)

ParabolicCylinderW[a_,z_]:=
  Module[{g1,g3,val},
    g1=Gamma[1/4+I a/2];g3=Gamma[3/4+I a/2];
    val=Exp[-I/4*z^2]*(2^(-3/4)*Sqrt[Abs[g1/g3]]*
      Hypergeometric1F1[1/4-I a/2,1/2,I/2*z^2]-
      2^(-1/4)*Sqrt[Abs[g3/g1]]*z*
      Hypergeometric1F1[3/4-I a/2,3/2,I/2*z^2]);
    If[Abs[Im[a]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                      ParabolicCylinderW1[a,z]                      *)
(**********************************************************************)

ParabolicCylinderW1[a_,z_]:=
  ParabolicCylinderW[a,z]/factorParabCylinSqrtK[a]/;
  (NumericQ[a] && NumericQ[z])

factorParabCylinSqrtK[a_]:=
  If[Re[a]<=0,
    Sqrt[Sqrt[1+Exp[2 Pi*a]]-Exp[Pi*a]],
    If[(1-(-1)^Ceiling[Abs[Im[a/2]]-1/2])/2===1,
    -Sqrt[factorParabCylinK[a]],
    Sqrt[factorParabCylinK[a]]]];

factorParabCylinK[a_]:=
  If[Re[a]<=0,
    Sqrt[1+Exp[2 Pi*a]]-Exp[Pi*a],
    If[(1-(-1)^Ceiling[Abs[Im[a]]-1/2])/2===1,
    -1/(Sqrt[1+Exp[2 Pi*a]]-Exp[Pi*a]),
    1/(Sqrt[1+Exp[2 Pi*a]]+Exp[Pi*a])]];

(**********************************************************************)
(*                      ParabolicCylinderW2[a,z]                      *)
(**********************************************************************)

ParabolicCylinderW2[a_,z_]:=
  factorParabCylinSqrtK[a]*ParabolicCylinderW[a,-z]/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                      ParabolicCylinderWC[a,z]                      *)
(**********************************************************************)

ParabolicCylinderWC[a_,z_]:=
  factorParabCylinSqrtK[a]*
    (ParabolicCylinderW[a,-z]+ParabolicCylinderW[a,z])/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                      ParabolicCylinderWS[a,z]                      *)
(**********************************************************************)

ParabolicCylinderWS[a_,z_]:=
  factorParabCylinSqrtK[a]*
    (ParabolicCylinderW[a,-z]-ParabolicCylinderW[a,z])/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                     ParabolicCylinderWEP[a,z]                      *)
(**********************************************************************)

ParabolicCylinderWEP[a_,z_]:=
  factorParabCylinSqrtK[a]*
    ((1+I)*ParabolicCylinderW[a,-z]+(1-I)*ParabolicCylinderW[a,z])/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                     ParabolicCylinderWEM[a,z]                      *)
(**********************************************************************)

ParabolicCylinderWEM[a_,z_]:=
  factorParabCylinSqrtK[a]*
    ((1-I)*ParabolicCylinderW[a,-z]+(1+I)*ParabolicCylinderW[a,z])/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                    ParabolicCylinderUPrime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderUPrime[a_,z_]:=
  Module[{r=Sqrt[2]},
    -2^(a/2-3/4)*Exp[-z^2/4]*
    (r*(2*a+1)*HermiteH[-a-3/2,z/r]+z*HermiteH[-a-1/2,z/r])]/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                    ParabolicCylinderVPrime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderVPrime[a_,z_]:=
  Module[{val},
    val=I/Gamma[1/2-a]*ParabolicCylinderUPrime[a,z]-
      I*Sqrt[2/Pi]*Exp[I*Pi*(a/2-1/4)]*ParabolicCylinderUPrime[-a,-I*z];
    If[Abs[Im[a]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (Not[numIntegQ[a]] && NumericQ[a] && NumericQ[z])

ParabolicCylinderVPrime[a_,z_]:=
  Module[{val},
    val=Gamma[a+1/2]/Pi*
      (Sin[Pi*a]*ParabolicCylinderUPrime[a,z]-ParabolicCylinderUPrime[a,-z]);
    If[Abs[Im[z]]>0,val,Re[val]]]/;
  (numIntegQ[a] && NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                    ParabolicCylinderWPrime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderWPrime[a_,z_]:=
  Module[{g1,g3,val},
    g1=Gamma[1/4+I a/2];g3=Gamma[3/4+I a/2];
    val=Exp[-I/4*z^2]/2*(2^(-3/4)*Sqrt[Abs[g1/g3]]*z*
      ((I+2*a)*Hypergeometric1F1[5/4-I*a/2,3/2,I/2*z^2]-
      I*Hypergeometric1F1[1/4-I*a/2,1/2,I/2*z^2])+
      2^(-1/4)*Sqrt[Abs[g3/g1]]*
      ((I*z^2-2)*Hypergeometric1F1[3/4-I*a/2,3/2,I/2*z^2]-
      (I+2*a/3)*z^2*Hypergeometric1F1[7/4-I*a/2,5/2,I/2*z^2]));
    If[Abs[Im[a]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                   ParabolicCylinderW1Prime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderW1Prime[a_,z_]:=
  ParabolicCylinderWPrime[a,z]/factorParabCylinSqrtK[a]/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                   ParabolicCylinderW2Prime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderW2Prime[a_,z_]:=
  -factorParabCylinSqrtK[a]*ParabolicCylinderWPrime[a,-z]/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                   ParabolicCylinderWCPrime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderWCPrime[a_,z_]:=
  factorParabCylinSqrtK[a]*
    (-ParabolicCylinderWPrime[a,-z]+ParabolicCylinderWPrime[a,z])/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                   ParabolicCylinderWSPrime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderWSPrime[a_,z_]:=
  factorParabCylinSqrtK[a]*
    (-ParabolicCylinderWPrime[a,-z]-ParabolicCylinderWPrime[a,z])/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                  ParabolicCylinderWEPPrime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderWEPPrime[a_,z_]:=
  factorParabCylinSqrtK[a]*
    (-(1+I)*ParabolicCylinderWPrime[a,-z]+
    (1-I)*ParabolicCylinderWPrime[a,z])/;
  (NumericQ[a] && NumericQ[z])

(**********************************************************************)
(*                  ParabolicCylinderWEMPrime[a,z]                    *)
(**********************************************************************)

ParabolicCylinderWEMPrime[a_,z_]:=
  factorParabCylinSqrtK[a]*
    (-(1-I)*ParabolicCylinderWPrime[a,-z]+
    (1+I)*ParabolicCylinderWPrime[a,z])/;
  (NumericQ[a] && NumericQ[z])



(**********************************************************************)
(*                      Hypergeometric1G1[a,c,z]                      *)
(**********************************************************************)

Hypergeometric1G1[a_,c_,z_]:=
  -Cot[c*Pi] Gamma[c] Hypergeometric1F1Regularized[a,c,z]-
  Pi*z^(1-c) Gamma[c] Hypergeometric1F1Regularized[a-c+1,2-c,z]/
  (Sin[a*Pi] Sin[c*Pi] Gamma[a] Gamma[c-a])/;(IntegerQ[c]===False)

Hypergeometric1G1[a_,c_,z_]:=
  Gamma[c]/Gamma[c-a]/Sin[a*Pi]*HypergeometricU[a,c,z]-
  Cot[a*Pi]*Hypergeometric1F1[a,c,z]/;(IntegerQ[c]===True)



(**********************************************************************)
(*                 CoulombWaveC[l,eta] & Common code                  *)
(**********************************************************************)

CoulombWaveC[l_,et_]:=
  2^l/Gamma[2 l+2]*Exp[coulombWavePi[l,et]-Pi*et/2]/;
  (NumericQ[l] && NumericQ[et])

coulombWavePi[l_,et_]:=
  If[Abs[et]>0 || Abs[Im[l]]>0,
    (LogGamma[l+1+I*et]+LogGamma[l+1-I*et])/2,
    Log[Abs[Gamma[l+1+I*et]]]]/;
  (NumericQ[l] && NumericQ[et])

CoulombWaveSigma[l_,et_]:=
  If[Abs[et]>0 || Abs[Im[l]]>0,
    (LogGamma[l+1+I*et]-LogGamma[l+1-I*et])/(2 I),
    Arg[Gamma[l+1+I*et]]]/;
  (NumericQ[l] && NumericQ[et])

CoulombWaveTheta[l_,et_,rh_]:=
  rh-et*Log[2*rh]-l*Pi/2+CoulombWaveSigma[l,et]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

precRegCWGH[et_,rh_]:=
  Module[{a=If[Abs[et]<1,2.36,0.11+2.25/Sqrt[Abs[et]]]},
    Min[216,Floor[16+20*Log[1+((Re[rh]/a)^2+(2*Im[rh]/a)^2)/1000]]]];

(**********************************************************************)
(*                      CoulombWaveF[l,eta,rho]                       *)
(**********************************************************************)

CoulombWaveF[l_,et_,rh_]:=
  Module[{val},
    val=2^l*Exp[coulombWavePi[l,et]-Pi*et/2]*rh^(l+1)*Exp[I*rh]*
      Hypergeometric1F1Regularized[l+1+I*et,2 l+2,-2 I*rh];
    If[Abs[Im[l]]>0 || Abs[Im[et]]>0 || Abs[Im[rh]]>0 || (Not[IntegerQ[l]] && Re[rh]<0),
      val,Re[val]]]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

(**********************************************************************)
(*                    CoulombWaveFPrime[l,eta,rho]                    *)
(**********************************************************************)

CoulombWaveFPrime[l_,et_,rh_]:=
  Module[{val},
    val=2^l*Exp[coulombWavePi[l,et]-Pi*et/2]*rh^l*Exp[I*rh]*
      ((l+1+I*rh)*Hypergeometric1F1Regularized[l+1+I*et,2 l+2,-2 I*rh]+
      2*(et-(l+1)*I)*rh*Hypergeometric1F1Regularized[l+2+I*et,2 l+3,-2 I*rh]);
    If[Abs[Im[l]]>0 || Abs[Im[et]]>0 || Abs[Im[rh]]>0 || (Not[IntegerQ[l]] && Re[rh]<0),
      val,Re[val]]]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

(**********************************************************************)
(*                      CoulombWaveG[l,eta,rho]                       *)
(**********************************************************************)

CoulombWaveG[l_,et_,rh_]:=
  Module[{prec=precRegCWGH[et,rh],lp,etp,rhp,val},
    Block[{$MinPrecision=prec},
      lp=SetPrecision[l,prec];
      etp=SetPrecision[et,prec];
      rhp=SetPrecision[rh,prec];
      val=If[Im[rh]>=0 || Re[rh]>0,
        (2*rhp)^(-I*etp)*(-2 I*rhp)^(lp+1+I*etp)*
          Exp[I (rhp-lp*Pi/2+CoulombWaveSigma[lp,etp])]*
          HypergeometricU[lp+1+I*etp,2 lp+2,-2 I*rhp]-I*CoulombWaveF[lp,etp,rhp],
        (2*rhp)^(I*etp)*(2 I*rhp)^(lp+1-I*etp)*
          Exp[-I (rhp-lp*Pi/2+CoulombWaveSigma[lp,etp])]*
          HypergeometricU[lp+1-I*etp,2 lp+2,2 I*rhp]+I*CoulombWaveF[lp,etp,rhp]];
      If[Abs[Im[l]]>0 || Abs[Im[et]]>0 || Abs[Im[rh]]>0 || Re[rh]<=0,
        val,Re[val]]]]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

(**********************************************************************)
(*                    CoulombWaveGPrime[l,eta,rho]                    *)
(**********************************************************************)

CoulombWaveGPrime[l_,et_,rh_]:=
  (CoulombWaveFPrime[l,et,rh]*CoulombWaveG[l,et,rh]-1)/CoulombWaveF[l,et,rh]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

(**********************************************************************)
(*                      CoulombWaveH1[l,eta,rho]                      *)
(**********************************************************************)

CoulombWaveH1[l_,et_,rh_]:=
  Module[{prec=precRegCWGH[et,rh],lp,etp,rhp},
    Block[{$MinPrecision=prec},
      lp=SetPrecision[l,prec];
      etp=SetPrecision[et,prec];
      rhp=SetPrecision[rh,prec];
      If[Im[rh]>=0 || Re[rh]>0,
        (2*rhp)^(-I*etp)*(-2 I*rhp)^(lp+1+I*etp)*
          Exp[I (rhp-lp*Pi/2+CoulombWaveSigma[lp,etp])]*
          HypergeometricU[lp+1+I*etp,2 lp+2,-2 I*rhp],
        (2*rhp)^(I*etp)*(2 I*rhp)^(lp+1-I*etp)*
          Exp[-I (rhp-lp*Pi/2+CoulombWaveSigma[lp,etp])]*
          HypergeometricU[lp+1-I*etp,2 lp+2,2 I*rhp]+
          2*I*CoulombWaveF[lp,etp,rhp]]]]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

(**********************************************************************)
(*                      CoulombWaveH2[l,eta,rho]                      *)
(**********************************************************************)

CoulombWaveH2[l_,et_,rh_]:=
  Module[{prec=precRegCWGH[et,rh],lp,etp,rhp},
    Block[{$MinPrecision=prec},
      lp=SetPrecision[l,prec];
      etp=SetPrecision[et,prec];
      rhp=SetPrecision[rh,prec];
      If[Im[rh]>=0 || Re[rh]>0,
        (2*rhp)^(-I*etp)*(-2 I*rhp)^(lp+1+I*etp)*
          Exp[I (rhp-lp*Pi/2+CoulombWaveSigma[lp,etp])]*
          HypergeometricU[lp+1+I*etp,2 lp+2,-2 I*rhp]-2*I*CoulombWaveF[lp,etp,rhp],
        (2*rhp)^(I*etp)*(2 I*rhp)^(lp+1-I*etp)*
          Exp[-I (rhp-lp*Pi/2+CoulombWaveSigma[lp,etp])]*
          HypergeometricU[lp+1-I*etp,2 lp+2,2 I*rhp]]]]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

(**********************************************************************)
(*                   CoulombWaveH1Prime[l,eta,rho]                    *)
(**********************************************************************)

CoulombWaveH1Prime[l_,et_,rh_]:=
  (CoulombWaveFPrime[l,et,rh]*CoulombWaveH1[l,et,rh]-1)/CoulombWaveF[l,et,rh]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])

(**********************************************************************)
(*                   CoulombWaveH2Prime[l,eta,rho]                    *)
(**********************************************************************)

CoulombWaveH2Prime[l_,et_,rh_]:=
  (CoulombWaveFPrime[l,et,rh]*CoulombWaveH2[l,et,rh]-1)/CoulombWaveF[l,et,rh]/;
  (NumericQ[l] && NumericQ[et] && NumericQ[rh])



(**********************************************************************)
(*               Generalized Sine and Cosine Integrals                *)
(**********************************************************************)

(* Si(v,z), Ci(v,z) *)
GeneralSinIntegralL[v_,z_]:=
  Module[{val},
    val=(Exp[v*Pi*I/2] acGamma1M[v,-I*z]-Exp[-v*Pi*I/2] acGamma1P[v,I*z])/(2 I);
    If[Abs[Im[v]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (N[v]=!=0. && Not[OddQ[v] && v<0] && NumericQ[v] && Precision[z]<Infinity)

GeneralSinIntegralL[v_,z_]:=
  SinIntegral[z]/;
  (N[v]===0. && Precision[z]<Infinity)

GeneralCosIntegralL[v_,z_]:=
  Module[{val},
    val=(Exp[v*Pi*I/2] acGamma1M[v,-I*z]+Exp[-v*Pi*I/2] acGamma1P[v,I*z])/2;
    If[Abs[Im[v]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (Not[EvenQ[v] && v<1] && NumericQ[v] && Precision[z]<Infinity)

(* si(v,z), ci(v,z) *)
GeneralSinIntegralS[v_,z_]:=
  Module[{val},
    val=(Exp[v*Pi*I/2] acGamma2M[v,-I*z]-Exp[-v*Pi*I/2] acGamma2P[v,I*z])/(2 I);
    If[Abs[Im[v]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (NumericQ[v] && Precision[z]<Infinity)

GeneralCosIntegralS[v_,z_]:=
  Module[{val},
    val=(Exp[v*Pi*I/2] acGamma2M[v,-I*z]+Exp[-v*Pi*I/2] acGamma2P[v,I*z])/2;
    If[Abs[Im[v]]>0 || Abs[Im[z]]>0,val,Re[val]]]/;
  (NumericQ[v] && Precision[z]<Infinity)

(* Analytic Continuation *)
acGamma1P[a_,z_]:=
  If[-Pi/2<=Arg[z]<Pi,Gamma[a,0,z],Exp[2 Pi I*a]*Gamma[a,0,z]];

acGamma1M[a_,z_]:=
  If[-Pi<=Arg[z]<Pi/2,Gamma[a,0,z],Exp[-2 Pi I*a]*Gamma[a,0,z]];

acGamma2P[a_,z_]:=
  If[-Pi/2<=Arg[z]<Pi,Gamma[a,z],
    Exp[2 Pi I*a]*Gamma[a,z]+(1-Exp[2 Pi I*a])*Gamma[a]];

acGamma2M[a_,z_]:=
  If[-Pi<=Arg[z]<Pi/2,Gamma[a,z],
    Exp[-2 Pi I*a]*Gamma[a,z]+(1-Exp[-2 Pi I*a])*Gamma[a]];



(**********************************************************************)
(*                   HyperErf[v,z], HyperErfi[v,z]                    *)
(**********************************************************************)

(* Hyper error function *)

HyperErf[v_,z_]:=
  Module[{per,n,val},
    per=2*Pi*I/Log[z];
    n=Floor[(Re[2 v/per]+1)/2];
    val=If[Re[z^v]>12 && Abs[Im[v]*Log[Abs[z]]+Re[v]*Arg[z]]<Pi/2,
      1-Exp[-z^v]*z^(1-v)/Gamma[1/v]*
        (1-Total[Map[(-1)^#*Pochhammer[1-1/v,#]/(z^(v*#))&,Range[15]]]),
      Exp[2 n*Pi I/v]*Gamma[1/v,0,z^v]/(v*Gamma[1+1/v])];
    If[Abs[Im[v]]>0 || Abs[Im[z]]>0 || Re[z]<0,val,Re[val]]]/;
  (Abs[z]>0 && NumericQ[v] && NumericQ[z])

HyperErf[v_,z_]:=0/;
  (Not[Abs[z]>0] && NumericQ[v] && NumericQ[z])

(* Hyper imaginary error function *)

HyperErfi[v_,z_]:=
  Module[{per,n,val},
    per=2*Pi*I/Log[z];
    n=Ceiling[Re[2 v/per]/2]-1;
    val=Exp[Pi*I (2 n+1)/v]*Gamma[1/v,0,-z^v]/(v*Gamma[1+1/v]);
    If[Abs[Im[v]]>0 || Abs[Im[z]]>0 || Re[z]<0,val,Re[val]]]/;
  (Not[Chop[Im[v]]===0 && Chop[Mod[v,2]]===0 && Re[z]<0] && 
   Abs[z]>0 && NumericQ[v] && NumericQ[z])

HyperErfi[v_,z_]:=
  -HyperErfi[v,-z]/;
  (Chop[Im[v]]===0 && Chop[Mod[v,2]]===0 && Re[z]<0 && 
   NumericQ[v] && NumericQ[z])

HyperErfi[v_,z_]:=0/;
  (Not[Abs[z]>0] && NumericQ[v] && NumericQ[z])



(**********************************************************************)
(*               HyperFresnelS[v,z], HyperFresnelC[v,z]               *)
(**********************************************************************)

(* HyperFresnelS *)

HyperFresnelS[v_,z_]:=
  Module[{c,val},
    c=(Pi*I/2)^(1/v);
    val=(consHypErfi[v,z]-consHypErf[v,z])/(2 I*c);
    If[Not[Abs[Im[v]]>0 || Abs[Im[z]]>0] && 
     (Re[z]>0 || Chop[v-Floor[v+1/2]]===0 && Re[v]>0),Re[val],val]]/;
  (N[v]=!=0. && NumericQ[v] && NumericQ[z])

HyperFresnelS[v_,z_]:=z/;(N[v]===0.)

(* HyperFresnelC *)

HyperFresnelC[v_,z_]:=
  Module[{c,val},
    c=(Pi*I/2)^(1/v);
    val=(consHypErfi[v,z]+consHypErf[v,z])/(2*c);
    If[Not[Abs[Im[v]]>0 || Abs[Im[z]]>0] && 
     (Re[z]>0 || Chop[v-Floor[v+1/2]]===0 && Re[v]>0),Re[val],val]]/;
  (Not[Chop[Im[v]]===0 && Chop[Mod[v+1/2,1]]===0 && Not[Abs[Im[z]]>0]] && 
   N[v]=!=0. && NumericQ[v] && NumericQ[z])

HyperFresnelC[v_,z_]:=
  Module[{expI,consV},
    expI=z*(-ExpIntegralE[1-1/v,-Pi*I/2*z^v]-
      ExpIntegralE[1-1/v,Pi*I/2*z^v])/(2 v);
    consV=(2/Pi)^(1/v)*Gamma[1+1/v]*Cos[\[Pi]/(2 v)];
    If[Re[z]>=0,Re[expI+consV],Re[expI-consV*Cos[\[Pi]/(2 v)]]]]/;
  (Chop[Im[v]]===0 && Chop[Mod[v+1/2,1]]===0 && Not[Abs[Im[z]]>0] && 
   N[v]=!=0. && NumericQ[v] && NumericQ[z])

HyperFresnelC[v_,z_]:=0/;(N[v]===0.)

(* HyperErf or HyperErfi in constant times arguments *)

consHypErf[v_,z_]:=
  Module[{c,per,n},
    c=(Pi*I/2)^(1/v);
    per=2*Pi*I/(Log[Pi*I/2]/v+Log[z]);
    n=Floor[(Re[2 v/per]+1)/2];
    If[Re[Pi*I/2*z^v]>12 && Abs[Im[v]*Log[Abs[c z]]+Re[v]*Arg[c z]]<Pi/2,
      Gamma[1+1/v]-Exp[-Pi*I/2*z^v]*(Pi*I/2)^(1/v-1)*z^(1-v)/v*
        (1-Total[Map[(-1)^#*Pochhammer[1-1/v,#]/((Pi*I/2)^#*z^(v*#))&,Range[15]]]),
      Exp[2 n*Pi I/v]*Gamma[1/v,0,Pi*I/2*z^v]/v]]/;
  (Abs[z]>0 && NumericQ[v] && NumericQ[z])

consHypErf[v_,z_]:=0/;
  (Not[Abs[z]>0] && NumericQ[v] && NumericQ[z])

consHypErfi[v_,z_]:=
  Module[{c,per,n},
    c=(Pi*I/2)^(1/v);
    per=2*Pi*I/(Log[Pi*I/2]/v+Log[z]);
    n=Floor[Re[2 v/per]/2];
    Exp[Pi*I (2 n+1)/v]*Gamma[1/v,0,-Pi*I/2*z^v]/v]/;
  (Not[Chop[Im[v]]===0 && Chop[Mod[v,2]]===0 && Re[z]<0] && 
   Abs[z]>0 && NumericQ[v] && NumericQ[z])

consHypErfi[v_,z_]:=
  -consHypErfi[v,-z]/;
  (Chop[Im[v]]===0 && Chop[Mod[v,2]]===0 && Re[z]<0 && 
   NumericQ[v] && NumericQ[z])

consHypErfi[v_,z_]:=0/;
  (Not[Abs[z]>0] && NumericQ[v] && NumericQ[z])



(**********************************************************************)
(*               GeneralErf[n,m,z], GeneralErfi[n,m,z]                *)
(**********************************************************************)

(* Common code *)

realOrCompGE[n_,m_,z_,val_]:=
  Module[{intQ,compQ,v},
    intQ[v_]:=(Chop[Im[v]]===0 && Chop[v-Floor[v+1/2]]===0);
    compQ[v_]:=(Abs[Im[v]]>0);
   If[compQ[z],val,
    If[intQ[n] && intQ[m],Re[val],
     If[compQ[n] || compQ[m],val,If[Re[z]<0,val,Re[val]]]]]]/;
  (NumericQ[n] && NumericQ[m] && NumericQ[z] && NumericQ[val])

(* Generalized error functions : n === Real && m === Real *)

GeneralErf[n_,m_,z_]:=
  Block[{$MinPrecision=16},
  Module[{np,mp,zp,prec=16,val},
    np=SetPrecision[n,prec];
    mp=SetPrecision[m,prec];
    zp=SetPrecision[z,prec];
    val=incGammaPower[(mp+1)/np,np,zp]/np;
    realOrCompGE[n,m,z,val]]]/;
  (Not[Abs[Im[n]]>0 || Abs[Im[m]]>0] && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralErfi[n_,m_,z_]:=
  Block[{$MinPrecision=16},
  Module[{np,mp,zp,prec=16,val},
    np=SetPrecision[n,prec];
    mp=SetPrecision[m,prec];
    zp=SetPrecision[z,prec];
    val=If[-Pi/n<Arg[z]<=Pi/n,
        (Exp[-(mp+1) Pi I/np]*GeneralErf[np,mp,zp*Exp[Pi I/np]]+
         Exp[(mp+1) Pi I/np]*GeneralErf[np,mp,zp*Exp[-Pi I/np]])/2,
        If[Pi/n<Arg[z]<Pi,
         Exp[2 Pi I (mp+1)/np]*GeneralErfi[np,mp,Exp[-2 Pi I/np]*zp],
         Exp[-2 Pi I (mp+1)/np]*GeneralErfi[np,mp,Exp[2 Pi I/np]*zp]]];
    realOrCompGE[n,m,z,val]]]/;
  (Not[Abs[Im[n]]>0 || Abs[Im[m]]>0 || 0<Re[n]<2] && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])

incGammaPower[a_,n_,z_]:=
  If[-Pi/n<Arg[z]<=Pi/n,Gamma[a,0,z^n],
   If[Pi/n<Arg[z]<Pi,
     Exp[2 Pi I*a]*incGammaPower[a,n,Exp[-2 Pi I/n]*z],
     Exp[-2 Pi I*a]*incGammaPower[a,n,Exp[2 Pi I/n]*z]]];

(* Generalized error functions : n =!= Real or m =!= Real *)

GeneralErf[n_,m_,z_]:=
  Module[{theta,sol,f,x,sN=10,shift,val},
    shift=Exp[I*Pi (1/2+Floor[sN*Arg[z]/Pi])/sN];
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-I*theta]*f'[x]-
      (Exp[I*theta]*x+shift)^m*Exp[-(Exp[I*theta]*x+shift)^n]==0,
      f[0]==GeneralErf[n,m,shift]},f,{x,Abs[z-shift]},
      MaxStepSize->0.001,MaxSteps->Infinity,StartingStepSize->0.001];
    val=f[Abs[z-shift]]/.sol[[1]];
    realOrCompGE[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0) && Abs[z]>1 && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralErfi[n_,m_,z_]:=
  Module[{theta,sol,f,x,sN=10,shift,val},
    shift=Exp[I*Pi (1/2+Floor[sN*Arg[z]/Pi])/sN];
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-I*theta]*f'[x]-
      (Exp[I*theta]*x+shift)^m*Exp[(Exp[I*theta]*x+shift)^n]==0,
      f[0]==GeneralErfi[n,m,shift]},f,{x,Abs[z-shift]},
      MaxStepSize->0.001,MaxSteps->Infinity,StartingStepSize->0.001];
    val=f[Abs[z-shift]]/.sol[[1]];
    realOrCompGE[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0 || 0<Re[n]<2) && Abs[z]>1 && 
   N[n]=!=0. && NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralErf[n_,m_,z_]:=
  Module[{sum=0,sumO=1,k=1,mf=-1,zf=z^n,fac=1,val},
    While[(sumO!=sum && k<100),sumO=sum;
      sum+=N[mf*zf/fac/(k+(m+1)/n)];
      mf*=(-1);zf*=z^n;k++;fac*=k];
    val=(1/(m+1)+sum/n)*z^(m+1);
    realOrCompGE[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0) && Abs[z]<=1 && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralErfi[n_,m_,z_]:=
  Module[{sum=0,sumO=1,k=1,zf=z^n,fac=1,val},
    While[(sumO!=sum && k<100),sumO=sum;
      sum+=N[zf/fac/(k+(m+1)/n)];
      zf*=z^n;k++;fac*=k];
    val=(1/(m+1)+sum/n)*z^(m+1);
    realOrCompGE[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0 || 0<Re[n]<2) && Abs[z]<=1 && 
   N[n]=!=0. && NumericQ[n] && NumericQ[m] && NumericQ[z])



(**********************************************************************)
(*           GeneralFresnelS[n,m,z], GeneralFresnelC[n,m,z]           *)
(**********************************************************************)

(* Common code *)

realOrCompGC[n_,m_,z_,val_]:=
  Module[{intQ,compQ,v},
    intQ[v_]:=(Chop[Im[v]]===0 && Chop[v-Floor[v+1/2]]===0);
    compQ[v_]:=(Abs[Im[v]]>0);
   If[compQ[z],val,
    If[(intQ[n] || intQ[n+1/2]) && intQ[m],Re[val],
     If[compQ[n] || compQ[m],val,If[Re[z]<0,val,Re[val]]]]]]/;
  (NumericQ[n] && NumericQ[m] && NumericQ[z] && NumericQ[val])

(* Generalized Fresnel functions : n === Real && m === Real *)

GeneralFresnelS[n_,m_,z_]:=
  Sin[1]/(m+1)*z^(m+1)/;(N[n]===0.)

GeneralFresnelS[n_,m_,z_]:=
  Module[{val},
    val=I*z^(1+m)*(ExpIntegralE[1-(1+m)/n,-I*z^n]-
      ExpIntegralE[1-(1+m)/n,I*z^n])/(2 n)+
      Gamma[(1+m)/n] Sin[(1+m) \[Pi]/(2 n)]/n;
    realOrCompGE[n,m,z,val]]/;
  (Not[Abs[Im[n]]>0 || Abs[Im[m]]>0] && -Pi/(2 n)<=Arg[z]<Pi/(2 n) && 
   N[n]=!=0. && NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralFresnelS[n_,m_,z_]:=
  If[Arg[z]<-Pi/(2 n),
    -Exp[-(m+1) Pi*I/n]*GeneralFresnelS[n,m,z*Exp[Pi*I/n]],
    -Exp[(m+1) Pi*I/n]*GeneralFresnelS[n,m,z*Exp[-Pi*I/n]]]/;
  (Not[Abs[Im[n]]>0 || Abs[Im[m]]>0] && Not[-Pi/(2 n)<=Arg[z]<Pi/(2 n)] && 
   N[n]=!=0. && NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralFresnelC[n_,m_,z_]:=
  Cos[1]/(m+1)*z^(m+1)/;(N[n]===0.)

GeneralFresnelC[n_,m_,z_]:=
  Module[{val},
    val=z^(1+m)*(-ExpIntegralE[1-(1+m)/n,-I*z^n]-
      ExpIntegralE[1-(1+m)/n,I*z^n])/(2 n)+
      Gamma[(1+m)/n]*Cos[(1+m) \[Pi]/(2 n)]/n;
    realOrCompGC[n,m,z,val]]/;
  (Not[Abs[Im[n]]>0 || Abs[Im[m]]>0] && -Pi/(2 n)<=Arg[z]<Pi/(2 n) && 
   N[n]=!=0. && NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralFresnelC[n_,m_,z_]:=
  If[Arg[z]<-Pi/(2 n),
    Exp[-(m+1) Pi*I/n]*GeneralFresnelC[n,m,z*Exp[Pi*I/n]],
    Exp[(m+1) Pi*I/n]*GeneralFresnelC[n,m,z*Exp[-Pi*I/n]]]/;
  (Not[Abs[Im[n]]>0 || Abs[Im[m]]>0] && Not[-Pi/(2 n)<=Arg[z]<Pi/(2 n)] && 
   N[n]=!=0. && NumericQ[n] && NumericQ[m] && NumericQ[z])

(* Generalized Fresnel functions : n =!= Real or m =!= Real *)

GeneralFresnelS[n_,m_,z_]:=
  Module[{theta,sol,f,x,sN=10,shift,val},
    shift=Exp[I*Pi (1/2+Floor[sN*Arg[z]/Pi])/sN];
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-I*theta]*f'[x]-
      (Exp[I*theta]*x+shift)^m*Sin[(Exp[I*theta]*x+shift)^n]==0,
      f[0]==GeneralFresnelS[n,m,shift]},f,{x,Abs[z-shift]},
      MaxStepSize->0.001,MaxSteps->Infinity,StartingStepSize->0.001];
    val=f[Abs[z-shift]]/.sol[[1]];
    realOrCompGE[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0) && Abs[z]>1 && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralFresnelC[n_,m_,z_]:=
  Module[{theta,sol,f,x,sN=10,shift,val},
    shift=Exp[I*Pi (1/2+Floor[sN*Arg[z]/Pi])/sN];
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-I*theta]*f'[x]-
      (Exp[I*theta]*x+shift)^m*Cos[(Exp[I*theta]*x+shift)^n]==0,
      f[0]==GeneralFresnelC[n,m,shift]},f,{x,Abs[z-shift]},
      MaxStepSize->0.001,MaxSteps->Infinity,StartingStepSize->0.001];
    val=f[Abs[z-shift]]/.sol[[1]];
    realOrCompGC[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0) && Abs[z]>1 && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralFresnelS[n_,m_,z_]:=
  Module[{sum=0,sumO=1,k=1,mf=-1,zf=z^(2 n),fac=6,val},
    While[(sumO!=sum && k<100),sumO=sum;
      sum+=N[mf*zf/fac/(k+1/2+(m+1)/(2 n))];
      mf*=(-1);zf*=z^(2 n);k++;fac*=(2 k (2 k+1))];
    val=(1/(n+m+1)+sum/(2 n))*z^(n+m+1);
    realOrCompGE[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0) && Abs[z]<=1 && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])

GeneralFresnelC[n_,m_,z_]:=
  Module[{sum=0,sumO=1,k=1,mf=-1,zf=z^(2 n),fac=2,val},
    While[(sumO!=sum && k<100),sumO=sum;
      sum+=N[mf*zf/fac/(k+(m+1)/(2 n))];
      mf*=(-1);zf*=z^(2 n);k++;fac*=(2 k (2 k-1))];
    val=(1/(m+1)+sum/(2 n))*z^(m+1);
    realOrCompGC[n,m,z,val]]/;
  ((Abs[Im[n]]>0 || Abs[Im[m]]>0) && Abs[z]<=1 && N[n]=!=0. && 
   NumericQ[n] && NumericQ[m] && NumericQ[z])



(**********************************************************************)
(*                         SinIntegralExp[z]                          *)
(**********************************************************************)

SinIntegralExp[z_]:=Pi/2-SinIntegral[Exp[z]];

(**********************************************************************)
(*                         CosIntegralExp[z]                          *)
(**********************************************************************)

CosIntegralExp[z_]:=
  -CosIntegral[Exp[z]]/;(N[-Pi]<Im[z]<=N[Pi])

CosIntegralExp[z_]:=
  CosIntegralExp[z+2 Pi I]+2 Pi I/;(Im[z]<=N[-Pi])

CosIntegralExp[z_]:=
  CosIntegralExp[z-2 Pi I]-2 Pi I/;(Im[z]>N[Pi])

(**********************************************************************)
(*                        SinIntegralRecip[z]                         *)
(**********************************************************************)

SinIntegralRecip[z_]:=z*Sin[1/z]-CosIntegral[1/z];

(**********************************************************************)
(*                        CosIntegralRecip[z]                         *)
(**********************************************************************)

CosIntegralRecip[z_]:=z*Cos[1/z]+SinIntegral[1/z]-Pi/2;



(**********************************************************************)
(*                         GoodwinStatonG[z]                          *)
(**********************************************************************)

GoodwinStatonG[z_]:=
  Module[{hyp,x,val},
    hyp=Re[z^2];
    x=If[-900<hyp<=-9,SetPrecision[z,20],z];
    val=If[hyp>-900,
        Exp[-x^2]/2*(Pi*Erfi[x]+ExpIntegralE[1,-x^2]+paiLogF[x]),
        -1/2*Total[Map[Gamma[(#+1)/2]*(-x)^(-#-1)&,Range[0,20]]]];
    If[Abs[Im[z]]>0 || Re[z]<=0,val,Re[val]]]/;
  (NumericQ[z])

paiLogF[z_]:=If[Im[z]>0,-Pi*I,Pi*I];

(**********************************************************************)
(*                        GoodwinStatonGExp[z]                        *)
(**********************************************************************)

GoodwinStatonGExp[z_]:=
  Module[{k=Floor[(1-Im[z]/Pi)/2]},
    GoodwinStatonG[Exp[z+2*Pi*I*k]]+2*Pi*I*k*Exp[-Exp[2 z]]]/;
  (NumericQ[z])



(**********************************************************************)
(*                      FresnelF[z], FresnelG[z]                      *)
(**********************************************************************)

FresnelF[z_]:=
  Module[{val},
    val=((1+I)*Exp[-Pi*I/2*z^2]*Erfc[(1-I)*Sqrt[Pi]/2*z]-
      (1-I)*Exp[Pi*I/2*z^2]*Erfc[(1+I)*Sqrt[Pi]/2*z])/(4*I);
    If[Abs[Im[z]]>0,val,Re[val]]]/;
  (NumericQ[z])

FresnelG[z_]:=
  Module[{val},
    val=((1+I)*Exp[-Pi*I/2*z^2]*Erfc[(1-I)*Sqrt[Pi]/2*z]+
      (1-I)*Exp[Pi*I/2*z^2]*Erfc[(1+I)*Sqrt[Pi]/2*z])/4;
    If[Abs[Im[z]]>0,val,Re[val]]]/;
  (NumericQ[z])

(**********************************************************************)
(*                         FresnelPatternF[z]                         *)
(**********************************************************************)

FresnelPatternF[z_]:=
  Module[{val},
    val=Erfc[(1+I)*Sqrt[Pi]*z/2]*Erfc[(1-I)*Sqrt[Pi]*z/2]/2;
    If[Abs[Im[z]]>0,val,Re[val]]]/;
  (NumericQ[z])



(**********************************************************************)
(*                         ImprovedOwenT[z,w]                         *)
(**********************************************************************)

ImprovedOwenT[z_,w_]:=
  Module[{phi1,phi2},
    phi1=(1+Erf[z/Sqrt[2]])/2;
    phi2=(1+Erf[z*w/Sqrt[2]])/2;
    If[Abs[w]<1,OwenT[z,w],
      (phi1+phi2)/2-phi1*phi2-OwenT[z*w,1/w]]]/;
  (Abs[w]>0 && (N[Re[w]]===0. && Im[w]>0 || Re[w]>0) && 
   NumericQ[z] && NumericQ[w])

ImprovedOwenT[z_,w_]:=
  -ImprovedOwenT[z,-w]/;
  (Abs[w]>0 && Not[N[Re[w]]===0. && Im[w]>0 || Re[w]>0] && 
   NumericQ[z] && NumericQ[w])

ImprovedOwenT[z_,w_]:=0/;
  (Not[Abs[w]>0] && NumericQ[z] && NumericQ[w])



(**********************************************************************)
(*                       ImprovedMarcumQ[m,z,w]                       *)
(**********************************************************************)

ImprovedMarcumQ[m_,z_,w_]:=
  Module[{prec=30,mp,zp,wp,sum=0,sumO=1,k=0,fac=1,bf=1,zf=1,w2f,w1f,minK,val},
  Block[{$MinPrecision=prec},
    mp=SetPrecision[m,prec];
    zp=SetPrecision[z,prec];
    wp=SetPrecision[w,prec];
    w2f=(wp^2)^mp;w1f=wp^(2 mp);
    minK=If[Chop[Im[m]]===0 && Chop[m-Floor[m+1/2]]===0 && Re[m]<1,
      Floor[1-m],0];
    While[(sumO!=sum && k<1000 || k<=minK),sumO=sum;
      sum+=(zf/bf/fac/w2f*((w2f-w1f)+w1f*GammaRegularized[mp+k,wp^2/2]));
      k++;
      fac*=k;bf*=2;zf*=(zp^2);w2f*=(wp^2);w1f*=(wp^2)];
    val=Exp[-zp^2/2]*sum;
  If[Abs[Im[m]]>0 || Abs[Im[z]]>0 || Abs[Im[w]]>0 || Re[w]<=0,val,Re[val]]]]/;
  (Abs[z]>0 && Abs[w]>0 && NumericQ[m] && NumericQ[z] && NumericQ[w])

ImprovedMarcumQ[m_,z_,w_]:=1/;
  (Not[Abs[w]>0] && Re[m]>0)

ImprovedMarcumQ[m_,z_,w_]:=
  (w^2)^(-m)*(((w^2)^m-w^(2 m))+w^(2 m)*GammaRegularized[m,w^2/2])/;
  (Not[Abs[z]>0] && (Abs[w]>0 || Re[m]<=0))



(**********************************************************************)
(*                          Voigt functions                           *)
(**********************************************************************)

VoigtU[z_,t_]:=
  Module[{xp,xm,val},
    xp=(1+I*z)/(2*Sqrt[t]);
    xm=(1-I*z)/(2*Sqrt[t]);
    val=Sqrt[Pi/(4*t)]*(Exp[xm^2]*Erfc[xm]+Exp[xp^2]*Erfc[xp])/2;
    If[Abs[Im[z]]>0 || Abs[Im[t]]>0 || Re[t]<=0,val,Re[val]]]/;
  (Abs[t]>0 && NumericQ[z] && NumericQ[t])

VoigtU[z_,t_]:=1/(1+z^2)/;
  (Not[Abs[t]>0] && NumericQ[z] && NumericQ[t])

VoigtV[z_,t_]:=
  Module[{xp,xm,val},
    xp=(1+I*z)/(2*Sqrt[t]);
    xm=(1-I*z)/(2*Sqrt[t]);
    val=Sqrt[Pi/(4*t)]*(Exp[xm^2]*Erfc[xm]-Exp[xp^2]*Erfc[xp])/(2*I);
    If[Abs[Im[z]]>0 || Abs[Im[t]]>0 || Re[t]<=0,val,Re[val]]]/;
  (Abs[t]>0 && NumericQ[z] && NumericQ[t])

VoigtV[z_,t_]:=z/(1+z^2)/;
  (Not[Abs[t]>0] && NumericQ[z] && NumericQ[t])

LineBroadeningH[a_,z_]:=
  Module[{val},
    val=(Exp[(a+I*z)^2]*Erfc[a+I*z]+Exp[(a-I*z)^2]*Erfc[a-I*z])/2;
    If[Abs[Im[z]]>0 || Abs[Im[a]]>0,val,Re[val]]]/;
  (NumericQ[a] && NumericQ[z])



(**********************************************************************)
(*                           HjertingH[n,z]                           *)
(**********************************************************************)

HjertingH[n_?EvenQ,z_]:=
  (-1)^(n/2)/(n!)*Exp[-z^2]*HermiteH[n,z]

HjertingH[n_?OddQ,z_]:=
  (-1)^((n-1)/2)/(n!)*(Exp[-z^2]*HermiteH[n,z]*Erfi[z]+
  2/Sqrt[Pi]*hjPolynomial[n,z])

hjPolyn[1,z_]:=-1

hjPolyn[n_/;(n>1 && OddQ[n]===True),z_]:=
  hjPolyn[n,z]=
  Collect[4*(n-2)*HermiteH[n-3,z]-2*z*HermiteH[n-2,z]+
    D[hjPolyn[n-2,z],{z,2}],z]

hjPolynomial[n_?OddQ,z_]:=
  hjPolyn[n,x]/.(x->z)



(**********************************************************************)

End[]

Protect[WhittakerN, WhittakerMPrime, WhittakerWPrime,
        HermiteH2, HermiteHPrime, HermiteH2Prime,
        HermiteHNormalized, HermiteH2Normalized,
        LaguerreL2, LaguerreLPrime, LaguerreL2Prime,
        LaguerreLNormalized, LaguerreL2Normalized,
        HydrogenElectronProbability, HydrogenElectronProbabilityR,
        HydrogenElectronProbabilityCartesian,
        ParabolicCylinderU, ParabolicCylinderV, ParabolicCylinderW,
        ParabolicCylinderW1, ParabolicCylinderW2,
        ParabolicCylinderWC, ParabolicCylinderWS,
        ParabolicCylinderWEP, ParabolicCylinderWEM,
        ParabolicCylinderUPrime, ParabolicCylinderVPrime,
        ParabolicCylinderWPrime, ParabolicCylinderW1Prime,
        ParabolicCylinderW2Prime, ParabolicCylinderWCPrime,
        ParabolicCylinderWSPrime, ParabolicCylinderWEPPrime,
        ParabolicCylinderWEMPrime,
        Hypergeometric1G1,
        CoulombWaveC, CoulombWaveSigma, CoulombWaveTheta,
        CoulombWaveF, CoulombWaveFPrime, CoulombWaveG,
        CoulombWaveGPrime, CoulombWaveH1, CoulombWaveH1Prime,
        CoulombWaveH2, CoulombWaveH2Prime,
        GeneralSinIntegralL, GeneralCosIntegralL,
        GeneralSinIntegralS, GeneralCosIntegralS,
        HyperErf, HyperErfi, HyperFresnelS, HyperFresnelC,
        GeneralErf, GeneralErfi, GeneralFresnelS, GeneralFresnelC,
        SinIntegralExp, CosIntegralExp, SinIntegralRecip, CosIntegralRecip,
        GoodwinStatonG, GoodwinStatonGExp, FresnelF, FresnelG, FresnelPatternF,
        ImprovedOwenT, ImprovedMarcumQ,
        VoigtU, VoigtV, LineBroadeningH, HjertingH]

EndPackage[]
